
%  Copyright (c) 2013
%      Oliver van Kaick <ovankaic@gmail.com>

%
% Get all the connected components from a set of possibly disconnected clusters
%
% Input:
%   - points: <n x 3> matrix of points
%   - gr: neighborhood relations of points, as returned by build_graph
%   - clusters: list of clusters, where clusters{i} is a list of all
%   points that are in cluster 'i'
%
% Output:
%   - cc: connected components
%   - cli: cluster index of each connected component
%

function [cc, cli] = conn_comp(points, gr, clusters)

    % Init output
    cc = {};
    cli = [];

    % Process each cluster
    for i = 1:length(clusters)
        %disp(['Cluster ' num2str(i) ' out of ' num2str(length(clusters))]);
        % Get all connected components in this cluster
        comp = get_conn_comp(points, gr, clusters{i});
        % Get a seed for each connected component based on centricity
        for j = 1:length(comp)
            %disp(['Component ' num2str(j) ' out of ' num2str(length(comp))]);

            % Add output
            cc{end+1} = comp{j};
            cli(end+1) = i;
        end
    end
end
