
%  Copyright (c) 2013
%      Noa Fish <noafish@post.tau.ac.il>

%%% cluster points into initial patches

%%% input:
% points: 3D point set
% normals: 3D normals corresponding to points
% k: number of neighbors for the KNN
% pnum: number of initial patches

%%% output:
% clusts: cell array containing the division into patches
% gr: KNN graph
% point_adj: point adjacency matrix
% max_elen: maximum edge length (to be used later for GC)
% outliers: outlier points


function [clusts, gr, point_adj, max_elen, outliers] = conv_clustering(points, normals, k, pnum)

npoints = size(points,1);

% calc KNN graph
[gr, point_adj, max_elen, outliers] = build_graph(points, k);

x = [];
y = [];
angs = [];

% collect angles between neighbors
for i=1:npoints
    
    x = [x; i];
    y = [y; i];
    angs = [angs; 1];
    
    neighs = gr{i};
    
    for j=1:length(neighs)
        
        neigh = neighs(j);
        n1 = normals(i,:);
        n2 = normals(neigh,:);
        
        ang = calc_angle(points(i, :), points(neigh, :), n1, n2);
        
        % distinguish between convex neighbors and concave neighbors
        if(ang >= 0.5)
            x = [x; i];
            y = [y; neigh];
            angs = [angs; 0.95];
        else
            x = [x; i];
            y = [y; neigh];
            angs = [angs; 0.01];
        end
    end
end

% adjacency matrix according to convexity
adj = sparse(x,y,angs,npoints,npoints);

% spectral clustering
[clusts, distortion] = gcut(adj, pnum);

% separate into connected components
[clusts, cli] = conn_comp(points, gr, clusts);


end



