
%  Copyright (c) 2013
%      Noa Fish <noafish@post.tau.ac.il>

%%% merge patches into weakly-convex components

%%% input:
% points: 3D point set
% normals: 3D normals corresponding to points
% patches: cell array containing the division into patches
% patch_adj_c: cell array containing patch adjacency
% gr: KNN graph
% idx: index of patch containing each point
% outliers: outlier points
% thresh: minimum visibility allowed
% mode: 0: strict mutual-visibility - each couple of patches to be merged
%          need to have visibility above the thresh
%       1: less trict mutual-visibility - visibility is computed between
%          super-patches therefore even if the visibility between two base
%          patches is below the thresh, the super-patches may still have
%          sufficient mutual visibility to be merged

%%% output:
% mergedu: cell array containing the grouped patches


function mergedu = convex_merge(points, normals, patches, patch_adj_c, gr, idx, outliers, thresh, mode)

ss_mode = 1;

% subsample points on each patch for the ray casting
patch_points = patch_subsample(points, outliers, ss_mode, patches);

% cast rays between neighboring patches to estimate visibility
[los_matrix, ilos_matrix] = patch_los_matrix_neigh(points, normals, patch_points, patch_adj_c, gr, outliers);

plosn = los_matrix ./ (los_matrix + ilos_matrix);

plosn2 = plosn;

merged = num2cell(1:length(patches));

[md, mind] = max(plosn2);
[mmd, mmind] = max(md);

while(mmd >= thresh)
    
    p1 = mind(mmind);
    p2 = mmind;
    
    m1 = merged{p1};
    m2 = merged{p2};
  
    merge = 1;
    
    if(mode == 0)
        
        for i=1:length(m1)
            for j=1:length(m2)
                p1 = m1(i);
                p2 = m2(j);
                if(los_matrix(p1,p2)+ilos_matrix(p1,p2) == 0)
                    [los, ilos] = patch_los_pair(points, normals, patch_points{p1}, patch_points{p2}, gr, outliers);
                    los_matrix(p1,p2) = los;
                    ilos_matrix(p1,p2) = ilos;
                    los_matrix(p2,p1) = los;
                    ilos_matrix(p2,p1) = ilos;

                else
                    los = los_matrix(p1,p2);
                    ilos = ilos_matrix(p1,p2);
                end
                losn = los / (los + ilos);
                if(losn < thresh)
                    merge = 0;
                    break;
                end
            end

        end
        
    else
    
        if(length(m1) ~= 1 || length(m2) ~= 1)

            if(length(m1) > 1)
                tmp = patches(m1);
                tmp = cell2mat(tmp);
                pnts1 = patch_subsample(points, outliers, ss_mode, {tmp});
                pnts1 = pnts1{1};
            else
                pnts1 = patch_points{p1};
            end

            if(length(m2) > 1)
                tmp = patches(m2);
                tmp = cell2mat(tmp);
                pnts2 = patch_subsample(points, outliers, ss_mode, {tmp});
                pnts2 = pnts2{1};
            else
                pnts2 = patch_points{p2};
            end

            [los, ilos] = patch_los_pair(points, normals, pnts1, pnts2, gr, outliers);
            losn = los / (los + ilos);

            if(losn < thresh)
                merge = 0;
            end 
        end
        
    end
    
    mp = [m1; m2];
    for i=1:length(mp)
        p = mp(i);
        plosn2(p,mp) = 0;
        if(merge)
            merged{p} = mp;
        end
    end
    
    
    [md, mind] = max(plosn2);
    [mmd, mmind] = max(md);
    
end

[mergedu, ~ ,~] = uniquecell(merged);


%show_points(points, patches2);



end
