
%  Copyright (c) 2013
%      Noa Fish <noafish@post.tau.ac.il>

function adj2 = detect_fake_neighbors(adj, seam_hist, sdf_thresh)


cthresh = 0.85;
adj2 = adj;

for i=1:size(adj,1)
    
    neigh = find(adj(i,:) ~= Inf);
    
    for l=1:length(neigh)
        
        j = neigh(l);
        
        if(j < i)
            continue;
        end
        
        neigh_j = find(adj(j,:) ~= Inf);
        
        common_neigh = intersect(neigh, neigh_j);
        
        % if two neighboring patches don't have any other common neighbors,
        % only check that the seam between them is convex enough
        
        if(isempty(common_neigh))
            conc = seam_hist(i,j,1);
            conv = seam_hist(i,j,3);
            conc_ratio = conc/conv;
            if(conc_ratio > cthresh)
                adj2(i,j) = Inf;
                adj2(j,i) = Inf;
            end
            
            continue;
        end
        
        % if two neighboring patches do share other common neighbors then
        % perform a series of checks to determine whether any of the pair
        % should not be merged
        
        for m=1:length(common_neigh)
            k = common_neigh(m);
            
            if(k < j)
                continue;
            end
            
            % total seam
            seam1 = sum(seam_hist(i,:,4));
            seam2 = sum(seam_hist(j,:,4));
            seam3 = sum(seam_hist(k,:,4));
            
            % length of seam between each pair
            seam12 = seam_hist(i,j,4);
            seam13 = seam_hist(i,k,4);
            seam23 = seam_hist(j,k,4);
            
            % convex-concave ratio of a pair's seam
            cratio12 = seam_hist(i,j,1) / seam_hist(i,j,3);
            cratio13 = seam_hist(i,k,1) / seam_hist(i,k,3);
            cratio23 = seam_hist(j,k,1) / seam_hist(j,k,3);
            
            % pair's current distance
            sd12 = adj2(i,j);
            sd13 = adj2(i,k);
            sd23 = adj2(j,k);
            
            if(cratio12 > cthresh)
                adj2(i,j) = Inf;
                adj2(j,i) = Inf;
                
                if(seam13 < seam23)
                    adj2(i,k) = Inf;
                    adj2(k,i) = Inf;
                else
                    adj2(j,k) = Inf;
                    adj2(k,j) = Inf;
                end
            end
            
            if(cratio13 > cthresh)
                adj2(i,k) = Inf;
                adj2(k,i) = Inf;
                
                if(seam12 < seam23)
                    adj2(i,j) = Inf;
                    adj2(j,i) = Inf;
                else
                    adj2(j,k) = Inf;
                    adj2(k,j) = Inf;
                end
            end
            
            if(cratio23 > cthresh)
                adj2(j,k) = Inf;
                adj2(k,j) = Inf;
                
                if(seam12 < seam13)
                    adj2(i,j) = Inf;
                    adj2(j,i) = Inf;
                else
                    adj2(i,k) = Inf;
                    adj2(k,i) = Inf;
                end
            end
            
            
            [~, mid] = min([seam12, seam13, seam23]);
            
            if(mid == 1 && sd13 > sdf_thresh && sd23 > sdf_thresh)
                adj2(i,j) = Inf;
                adj2(j,i) = Inf;
            end
            
            if(mid == 2 && sd12 > sdf_thresh && sd23 > sdf_thresh)
                adj2(i,k) = Inf;
                adj2(k,i) = Inf;
            end
            
            if(mid == 3 && sd12 > sdf_thresh && sd13 > sdf_thresh)
                adj2(j,k) = Inf;
                adj2(k,j) = Inf;
            end
            
            
            if(~valid_seam(seam1, seam2, seam12))
                adj2(i,j) = Inf;
                adj2(j,i) = Inf;
            end
            
            if(~valid_seam(seam1, seam3, seam13))
                adj2(i,k) = Inf;
                adj2(k,i) = Inf;
            end
            
            if(~valid_seam(seam2, seam3, seam23))
                adj2(j,k) = Inf;
                adj2(k,j) = Inf;
            end
            
            
        end
    end
end

end


function valid = valid_seam(seam1, seam2, seam12)

thresh = 10;
valid = 1;

if(seam12 < thresh)
    valid = 0;
end


end


