
%  Copyright (c) 2013
%      Noa Fish <noafish@post.tau.ac.il>

%%% merge flat patches with a neighboring patch or a facing patch

%%% input:
% flats: id's of flat patches
% ang_hist: seam sets
% flat_fronters: id's of patches that are sufficiently in front of a patch
%                as given by the patch_sdf calculation
% info: contains data structures such as 'points' and 'normals'
% ppoints: cell array containing candidate points for each patch
% idx: index of patch containing each point

%%% output:
% merged: cell array containing the grouped patches


function merged = merge_flat_patches(flats, ang_hist, flat_fronters, info, ppoints, idx)

mergedt = num2cell(1:size(ang_hist,1));

for i=1:length(flats)
    
    flat = flats(i);
    m1 = mergedt{flat};
    if(length(m1) > 1)
        continue;
    end
    
    ff = flat_fronters{flat};
    
    seam_len = ang_hist(flat,:,4);
    sum_seam_len = sum(seam_len);
    sum_seam_len = max(sum_seam_len, 1);
    non_z = find(seam_len >= 0.1*sum_seam_len);
    
    % if a flat patch doesn't have a neighbor with which it shares a
    % sufficiently long seam, check whether it faces a patch that was
    % hit by this patch's rays at least 50% of the time
    if(isempty(non_z))
        if(~isempty(ff) && ff(2,1) >= 0.5)
            mergedt = update_merge(flat, ff(1,1), mergedt);
        end
        
        continue;
    end
    
    chosen = -1;
    
    [seam_len, sind] = sort(seam_len, 'descend');
    valid = find(seam_len > 0.1*sum_seam_len);
    valid_ind = sind(valid);
    ratios = [];
    
    % look for the neighbor with the longest seam that is also either a good hit
    % for the rays of this patch, or shares a convex seam with it
    for j=1:length(valid_ind)
        ffc = find(ff(1,:) == valid_ind(j));
        if(~isempty(ffc) && ff(2,ffc) >= 0.5)
            chosen = valid_ind(j);
            break;
        end
        conc = ang_hist(flat,valid_ind(j),1);
        conv = ang_hist(flat,valid_ind(j),3);
        ratio = conv / conc;
        if(ratio == Inf)
            chosen = valid_ind(j);
            break;
        end
        ratios = [ratios; ratio];
    end
    
    % if a good neighbor hasn't been found, look for the neighbor with the
    % best convex-concave ratio that has a mutual visibility of at least
    % 0.4 with the flat patch
    if(chosen == -1)
        [sratios, rind] = sort(ratios, 'descend');
        
        for j=1:length(sratios)
            if(sratios(j) < 1)
                break;
            end
            ind = valid_ind(rind(j));
            [los, ilos] = patch_los_pair(info.points, info.normals, ppoints{ind}, ppoints{flat}, info.gr, info.outliers);
            nlos = los / (los + ilos);
            if(nlos >= 0.4)
                chosen = ind;
                break;
            end
        end
        
        if(chosen == -1)
            continue;
        end
    end
    
    mergedt = update_merge(flat, chosen, mergedt);
      

end

[merged, ~ ,~] = uniquecell(mergedt);

end


function merged = update_merge(i, j, merged)

m1 = merged{i};
m2 = merged{j};

m3 = [m1 m2];
    
for k=1:length(m3)
    merged{m3(k)} = m3;
end

end



