
%  Copyright (c) 2013
%      Oliver van Kaick <ovankaic@gmail.com>


% Selection of input models
%select_shapes;
run_graphcut = 1;
eliminate_points = 1;

% Process each file
all_tm = [];
for fin = 1:length(filenames)
    % Print information and set filenames
    
    mat_file = [session_output_dirname filesep 'session_' filenames{fin} '.mat'];
    if(exist(mat_file, 'file'))
        %continue;
    end
    
    disp(filenames{fin});
    filename = [dirname filesep filenames{fin}];
    part = strsplit('.', filenames{fin});
    tm = [];

    % Load file
    disp('Reading input'); drawnow('update');
    if point_cloud
        if strcmp(part{end}, 'obj')
            P = read_obj(filename);
        elseif strcmp(part{end}, 'ply')
            P = read_ply(filename);
        else
            disp('Invalid input format');
        end
        points = P.v;
        normals = P.n;
        npoints = size(points,1);
        if eliminate_points
            [points, normals] = eliminate_closeby_points(points, normals, 0.005);
            npoints = size(points,1);
        end
    else
        [vertices, faces] = read_off_file(filename);
        [points, normals] = mesh_to_point_cloud(vertices, faces);
        npoints = size(points,1);
        if(size(points,1) < 10000)
            orig_points = points;
            orig_normals = normals;
            [points, normals] = add_points_to_mesh(vertices, faces, points, normals);
        end
    end

    % Subsample large point clouds
    if 0
    if subsample || (size(points, 1) > 50000)
        disp('Subsampling input'); drawnow('update');
        old_points = points;
        old_normals = normals;
        [selected, points, normals] = subsample_point_cloud(points, normals, 10000);
    end
    end

    %%%%%%%%%% initial segmentation into patches %%%%%%%%%%
    disp('Building patches'); drawnow('update');
    tic;
    init_seg;
    tm(end+1) = toc;
    
    %show_points(points, patches);

    %%%%%%%%%% convex merging %%%%%%%%%%
    disp('Convex merging'); drawnow('update');
    tic;
    conv_merge;
    tm(end+1) = toc;
    
    %show_points(points, conv_patches);

    %%%%%%%%%% sdf merging %%%%%%%%%%
    disp('SDF merging'); drawnow('update');
    sdf_merge;
    tic;
    tm(end+1) = toc;

    %show_points(points, final_patches);
    
    %%%%%%%%%% graphcut %%%%%%%%%%
    if run_graphcut
    disp('Graphcut'); drawnow('update');
    tic;
    run_gc;
    tm(end+1) = toc;
    end

    idxs = idx;
    conv_idxs = conv_idx;
    final_idxs = final_idx;
    graphcut_idxs = graphcut_idx;
    
    if(npoints ~= size(points,1))
        
        idxs = clean_seg(idxs(1:npoints));
        conv_idxs = clean_seg(conv_idxs(1:npoints));
        final_idxs = clean_seg(final_idxs(1:npoints));
        graphcut_idxs = clean_seg(graphcut_idxs(1:npoints));
    end
    
    % Write output
    disp('Writing output'); drawnow('update');
    if point_cloud
        output_filename = [output_dirname filesep part{1:(end-1)} '-patches.ply'];
        write_ply(points, normals, clustering_to_coloring(idxs), output_filename);
        output_filename = [output_dirname filesep part{1:(end-1)} '-merged1.ply'];

        write_ply(points, normals, clustering_to_coloring(conv_idxs), output_filename);
        output_filename = [output_dirname filesep part{1:(end-1)} '-merged2.ply'];

        write_ply(points, normals, clustering_to_coloring(final_idxs), output_filename);
        if run_graphcut
        output_filename = [output_dirname filesep part{1:(end-1)} '-graphcut.ply'];

        write_ply(points, normals, clustering_to_coloring(graphcut_idxs), output_filename);
        end
    else
        output_filename = [output_dirname filesep part{1:(end-1)} '-patches.off'];
        write_color_off_file(output_filename, vertices, faces, clustering_to_coloring(idxs));
        output_filename = [output_dirname filesep part{1:(end-1)} '-merged1.off'];
        write_color_off_file(output_filename, vertices, faces, clustering_to_coloring(conv_idxs));
        output_filename = [output_dirname filesep part{1:(end-1)} '-merged2.off'];
        write_color_off_file(output_filename, vertices, faces, clustering_to_coloring(final_idxs));
        if run_graphcut
        output_filename = [output_dirname filesep part{1:(end-1)} '-graphcut.off']; 
        write_color_off_file(output_filename, vertices, faces, clustering_to_coloring(graphcut_idxs));
        end
    end

    % Write segmentation
    output_filename = [output_dirname filesep part{1:(end-1)} '-patches.seg'];
    seg = idxs - 1;
    write_seg(seg, output_filename);

    output_filename = [output_dirname filesep part{1:(end-1)} '-merged1.seg'];
    seg = conv_idxs - 1;
    write_seg(seg, output_filename);

    output_filename = [output_dirname filesep part{1:(end-1)} '-merged2.seg'];
    seg = final_idxs - 1;
    write_seg(seg, output_filename);

    if run_graphcut
    output_filename = [output_dirname filesep part{1:(end-1)} '-graphcut.seg'];
    seg = graphcut_idxs - 1;
    write_seg(seg, output_filename);
    end
    

    % Save session
    save([session_output_dirname filesep 'session_' filenames{fin} '.mat']);

    % Save time
    all_tm(end+1, :) = tm;
end
