
%  Copyright (c) 2013
%      Noa Fish <noafish@post.tau.ac.il>

%%%%%%%%%% merge segments according to SDF %%%%%%%%%%

thresh = 0.12;

if(length(conv_patches) == 1)
    final_patches = conv_patches;
    final_idx = conv_idx;
    return;
end

info = struct('points', points, 'normals', normals, 'outliers', outliers, 'gr', {gr});

sdf_patches = conv_patches;
sdf_idx = conv_idx;
sdf_patch_adj = conv_patch_adj;
sdf_patch_adj_c = conv_patch_adj_c;
sdf_ang_hist = conv_ang_hist;

% subsample points on each patch for SDF value measurement
patch_points2 = patch_subsample(points, normals, 2, conv_patches);

% perform SDF calc
conv_str = patch_sdf(points, normals, patch_points2, 1:length(conv_patches), outliers, conv_idx, 0);
sdf_str = conv_str;

flats = sdf_str.flats;

sdf_norm = sdf_str.sdf_norm;

% create distance matrix
patch_dists = earthMoversDistances(sdf_norm',sdf_norm');
patch_dists2 = patch_dists / max(max(patch_dists));
patch_dists2 = patch_dists2 + eye(size(patch_dists2));

% create adjacency-distance matrix
adj = sdf_patch_adj;
adj(adj == 0) = Inf;
adj_dists = patch_dists2 .* adj;
adj_dists(isnan(adj_dists)) = Inf;
adj_dists(sdf_ang_hist(:,:,4) < 0) = Inf;

% some neighboring patches may have similar histograms but be cosidered far
% by EMD standards
adj_dists = detect_good_neighbors(adj_dists, sdf_norm);
% find neighboring patches that should not be merged despite similar
% histogram
adj_dists = detect_fake_neighbors(adj_dists, sdf_ang_hist, thresh);

% exclude flat patches from current merge step
adj_dists(flats,:) = Inf;
adj_dists(:,flats) = Inf;

% merge patches according to adj_dists and thresh
merged = num2cell(1:size(sdf_norm,1));
merge = 1;
tmp_adj_dists = adj_dists;

while(merge)
    [tmp_adj_dists, merged, merge] = merge_patches(tmp_adj_dists, merged, thresh);
end

[mergedu, ~ ,~] = uniquecell(merged);

%prev_sdf_patches = sdf_patches;

[sdf_patches, sdf_idx, sdf_patch_adj, sdf_patch_adj_c, sdf_ang_hist, sdf_str] = merge_and_update(sdf_patches, sdf_idx, mergedu, sdf_patch_adj_c, sdf_ang_hist, sdf_str, info);


% if there are flat patches, try to merge them with other patches and then
% run another round of SDF merge

if(~isempty(sdf_str.flats))

    while(~isempty(sdf_str.flats))
        ppoints = patch_subsample(points, normals, 1, sdf_patches);
        mergedf = merge_flat_patches(sdf_str.flats, sdf_ang_hist, sdf_str.flats2, info, ppoints, sdf_idx);
        if(length(mergedf) == length(sdf_patches))
            % if there hasn't been any change (no merge), break
            break;
        else
            [sdf_patches, sdf_idx, sdf_patch_adj, sdf_patch_adj_c, sdf_ang_hist, sdf_str] = merge_and_update(sdf_patches, sdf_idx, mergedf, sdf_patch_adj_c, sdf_ang_hist, sdf_str, info);
        end
    end
    
    
    sdf_norm = sdf_str.sdf_norm;

    patch_dists = earthMoversDistances(sdf_norm',sdf_norm');
    patch_dists2 = patch_dists / max(max(patch_dists));
    patch_dists2 = patch_dists2 + eye(size(patch_dists2));


    adj = sdf_patch_adj;
    adj(adj == 0) = Inf;
    adj_dists = patch_dists2 .* adj;
    adj_dists(isnan(adj_dists)) = Inf;
    adj_dists(sdf_ang_hist(:,:,4) < 0) = Inf;

    adj_dists = detect_good_neighbors(adj_dists, sdf_norm);
    adj_dists = detect_fake_neighbors(adj_dists, sdf_ang_hist, thresh);
    %adj_dists(flats,:) = Inf;
    %adj_dists(:,flats) = Inf;

    merged = num2cell(1:size(sdf_norm,1));
    merge = 1;

    tmp_adj_dists = adj_dists;

    while(merge)
        [tmp_adj_dists, merged, merge] = merge_patches(tmp_adj_dists, merged, thresh);
    end

    [mergedu, ~ ,~] = uniquecell(merged);

    %prev_sdf_patches = sdf_patches;
    
    [sdf_patches2, sdf_idx2, sdf_patch_adj2, sdf_patch_adj_c2, sdf_ang_hist2, sdf_str2] = merge_and_update(sdf_patches, sdf_idx, mergedu, sdf_patch_adj_c, sdf_ang_hist, sdf_str, info);
    
else
    
    sdf_patches2 = sdf_patches;
    sdf_idx2 = sdf_idx;
    sdf_patch_adj2 = sdf_patch_adj;
    sdf_patch_adj_c2 = sdf_patch_adj_c;
    sdf_ang_hist2 = sdf_ang_hist;
    sdf_str2 = sdf_str;
    
end


% in order to blend in patches that may have been left un-merged due to
% various inaccuracies, look at each patch's facing patches as given by
% patch_sdf and allow merging with such patches that share a convex seam
patch_points3 = patch_subsample(points, normals, 2, sdf_patches2);
sdf_str2 = patch_sdf(points, normals, patch_points3, 1:length(sdf_patches2), outliers, sdf_idx2, 0, sdf_str2);
ffronts = sdf_str2.flats2;
ff_len = cellfun(@length, ffronts);
nzlen = find(ff_len > 0);
merged = num2cell(1:length(sdf_patches2));

for i=1:length(nzlen)
    ind = nzlen(i);
    ff = ffronts{ind};
    conc = sdf_ang_hist2(ind, ff(1,:),1);
    conv = sdf_ang_hist2(ind, ff(1,:),3);
    ratio = conv ./ conc;
    tmp = find(ratio > 1);
    for k=1:length(tmp)
        mt = ff(1,tmp(k));
        if(sdf_patch_adj2(ind, mt) == 1)
            merg = [merged{ind} merged{mt}];
            for j=1:length(merg)
                merged{merg(j)} = merg;
            end
            break;
        end
    end

end

[mergedu, ~ ,~] = uniquecell(merged);
if(length(mergedu) < length(sdf_patches2))
    [sdf_patches2, sdf_idx2, sdf_patch_adj2, sdf_patch_adj_c2, sdf_ang_hist2, sdf_str2] = merge_and_update(sdf_patches2, sdf_idx2, mergedu, sdf_patch_adj_c2, sdf_ang_hist2, sdf_str2, info);
end

final_patches = sdf_patches2;
final_idx = sdf_idx2;
final_patch_adj = sdf_patch_adj2;
final_patch_adj_c = sdf_patch_adj_c2;
final_ang_hist = sdf_ang_hist2;
final_str = sdf_str2;


%show_points(points, final_patches);




